package net.wanji.opt.servicev2.implv2;

import com.alibaba.nacos.api.utils.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.dao.entity.GreenwaveHistPO;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.dao.mapper.GreenwaveHistMapper;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.opt.common.enums.EventInfoTypeEnum;
import net.wanji.opt.common.enums.GreenBeltDirEnum;
import net.wanji.opt.dao.mapper.GreenWaveRealTimeMapperV2Mapper;
import net.wanji.opt.dao.mapper.HoloEventMapper;
import net.wanji.opt.dao.mapper.StrategyCrossResultMapper;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.servicev2.TrendServiceV2;
import net.wanji.opt.vo2.*;
import net.wanji.opt.vo2.dto.CrossLastOptResultDTO;
import net.wanji.opt.vo2.dto.GreenLastOptResultDTO;
import net.wanji.opt.vo2.dto.GreenOptDTO;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2025/03/06 18:44
 */
@Service
@Slf4j
public class TrendServiceV2Impl implements TrendServiceV2 {

    @Resource
    private HoloEventMapper holoEventMapper;
    @Resource
    private StrategyCrossResultMapper strategyCrossResultMapper;
    @Resource
    private CrossDataHistMapper crossDataHistMapper;
    @Resource
    private GreenWaveRealTimeMapperV2Mapper greenWaveRealTimeMapperV2Mapper;
    @Resource
    private GreenwaveHistMapper greenwaveHistMapper;
    @Resource
    private StrategyGreenOptHistMapper greenOptHistMapper;

    @Override
    public List<CrossGreenStatusTimeRateVO> crossGreenStatusTimeRate() {
        List<CrossGreenStatusTimeRateVO> crossGreenStatusTimeRateVOS = holoEventMapper.selectCrossGreenStatusTimeRate();
        if (!CollectionUtils.isEmpty(crossGreenStatusTimeRateVOS)) {
            Map<String, CrossGreenStatusTimeRateVO> map = crossGreenStatusTimeRateVOS.stream().collect(Collectors.toMap(CrossGreenStatusTimeRateVO::getType, crossGreenStatusTimeRateVO -> crossGreenStatusTimeRateVO));
            for (EventInfoTypeEnum typeEnum : EventInfoTypeEnum.values()) {
                String eventType = typeEnum.getEventType();
                if (!map.containsKey(eventType)) {
                    CrossGreenStatusTimeRateVO vo = new CrossGreenStatusTimeRateVO();
                    vo.setType(eventType);
                    vo.setCount(0);
                    crossGreenStatusTimeRateVOS.add(vo);
                }
            }

        }
        return  crossGreenStatusTimeRateVOS;
    }

    @Override
    public List<CrossOptInfoVO> crossOptInfoList() throws Exception {
        List<CrossOptInfoVO> results = new ArrayList<>();
        try {
            results = new ArrayList<>();
            List<CrossOptInfoVO> resultList = strategyCrossResultMapper.selectAICrossListV2();
            long start = DateUtil.getCurrentTimeSecond();
            long end = start + 5 * 60;
            LambdaQueryWrapper<CrossDataHistPO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.between(CrossDataHistPO::getBatchTime, start, end);
            List<CrossDataHistPO> crossDataHistPOS = crossDataHistMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(resultList)) {
                for (CrossOptInfoVO entity : resultList) {
                    CrossOptInfoVO result = new CrossOptInfoVO();
                    String crossId = entity.getCrossId();
                    Double trafficIndex = entity.getTrafficIndex();
                    Integer queueLength = entity.getQueueLength();
                    String location = entity.getLocation();
                    location = location.replace("POINT(", "").replace(" ", ",").replace(")", "");
                    BeanUtils.copyProperties(entity, result);
                    result.setLocation(location);
                    for (CrossDataHistPO crossDataHistPO : crossDataHistPOS) {
                        String lastWeekCrossId = crossDataHistPO.getCrossId();
                        Double lastWeekTrafficIndex = crossDataHistPO.getTrafficIndex();
                        Double lastWeekQueueLength = crossDataHistPO.getQueueLength();
                        if (StringUtils.equals(crossId, lastWeekCrossId)) {
                            if (trafficIndex > lastWeekTrafficIndex) {
                                result.setTrafficIndexUpDown(1);
                            } else {
                                result.setTrafficIndexUpDown(0);
                            }
                            if (queueLength > lastWeekQueueLength) {
                                result.setQueueLengthUpDown(1);
                            } else {
                                result.setQueueLengthUpDown(0);
                            }
                            results.add(result);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("态势监测-区域体检-雷达图-查询失败:{}", e);
            throw new Exception(e);
        }
        return results;
    }

    @Override
    public List<GreenOptInfoVO> greenOptInfoList() throws Exception {
        List<GreenOptInfoVO> results = new ArrayList<>();
        List<GreenOptDTO> greenOptDTOS = greenWaveRealTimeMapperV2Mapper.selectGreenOptInfoList();
        long start = DateUtil.getCurrentTimeSecond();
        long end = start + 5 * 60;
        LambdaQueryWrapper<GreenwaveHistPO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.between(GreenwaveHistPO::getBatchTime, start, end);
        List<GreenwaveHistPO> greenwaveHistPOS = greenwaveHistMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(greenOptDTOS)) {
            for (GreenOptDTO greenOptDTO : greenOptDTOS) {
                Integer greenId = greenOptDTO.getGreenId();
                Double trafficIndex = greenOptDTO.getTrafficIndex();
                Integer speed = greenOptDTO.getSpeed();
                Integer travelTime = greenOptDTO.getTravelTime();
                greenOptDTO.setStrategyName("效率提升");
                String roadDirection = greenOptDTO.getRoadDirection();
                greenOptDTO.setDirectionName(GreenBeltDirEnum.getDesc(roadDirection));
                for (GreenwaveHistPO greenwaveHistPO : greenwaveHistPOS) {
                    Integer curGreenId = greenwaveHistPO.getGreenId();
                    String curDirection = greenwaveHistPO.getRoadDirection();
                    Double curTrafficIndex = greenwaveHistPO.getTrafficIndex();
                    Double curSpeed = Objects.nonNull(greenwaveHistPO.getSpeed()) ? greenwaveHistPO.getSpeed() : 0;
                    Integer curTravelTime = greenwaveHistPO.getTrvalTime();
                    if (Objects.equals(greenId, curGreenId) && StringUtils.equals(roadDirection, curDirection)) {

                        if (trafficIndex > curTrafficIndex) {
                            greenOptDTO.setTrafficIndexUpDown(1);
                        } else {
                            greenOptDTO.setTrafficIndexUpDown(0);
                        }
                        if (speed > curSpeed.intValue()) {
                            greenOptDTO.setSpeedUpDown(1);
                        } else {
                            greenOptDTO.setSpeedUpDown(0);
                        }
                        if (travelTime > curTravelTime) {
                            greenOptDTO.setTravelUpDown(1);
                        } else {
                            greenOptDTO.setTravelUpDown(0);
                        }
                    }

                }
            }
            Map<Integer, List<GreenOptDTO>> greenIdMap = greenOptDTOS.stream().collect(Collectors.groupingBy(GreenOptDTO::getGreenId));
            for (Map.Entry<Integer, List<GreenOptDTO>> entry : greenIdMap.entrySet()) {
                GreenOptInfoVO greenOptInfoVO = new GreenOptInfoVO();
                Integer greenId = entry.getKey();
                List<GreenOptDTO> value = entry.getValue();
                GreenOptDTO greenOptDTO = value.get(0);
                greenOptInfoVO.setGreenId(greenId);
                greenOptInfoVO.setGreenName(greenOptDTO.getGreenName());
                greenOptInfoVO.setDetails(value);
                results.add(greenOptInfoVO);
            }
        }
        return results;
    }

    @Override
    public List<OptMonitoringVO> greenOptMonitoringList() throws Exception {
        List<OptMonitoringVO> optMonitoringVOS = holoEventMapper.selectGreenOptMonitoringList();
        List<GreenLastOptResultDTO> resultDTOS = greenOptHistMapper.selectLastGreenOptResultList();
        long current = System.currentTimeMillis();
        optMonitoringVOS.forEach(optMonitoringVO -> {
            String id = optMonitoringVO.getId();
            optMonitoringVO.setTypeDesc(EventInfoTypeEnum.getOptDesc(optMonitoringVO.getType()));
            if (!CollectionUtils.isEmpty(resultDTOS)) {
                for (GreenLastOptResultDTO resultDTO : resultDTOS) {
                    Integer greenId = resultDTO.getGreenId();
                    if (Objects.equals(greenId, Integer.valueOf(id))) {
                        Integer controlMethod = resultDTO.getControlMethod();
                        Date optTime = resultDTO.getOptTime();
                        Integer duration = resultDTO.getDuration();
                        if (Objects.equals(-1, controlMethod)) {
                            optMonitoringVO.setOptStatus(0);
                            //optMonitoringVO.setOptStatusDesc("");
                        } else if (current < optTime.getTime() + duration) {
                            optMonitoringVO.setOptStatus(1);
                            optMonitoringVO.setOptStatusDesc("优化中");
                        }
                    }
                }
            }
        });
        return optMonitoringVOS;
    }

    @Override
    public List<OptMonitoringVO> crossOptMonitoringList() throws Exception {
        List<OptMonitoringVO> optMonitoringVOS = holoEventMapper.selectCrossOptMonitoringList();
        List<CrossLastOptResultDTO> optResultDTOS = strategyCrossResultMapper.selectLastOneCrossOptList();
        long current = System.currentTimeMillis();
        optMonitoringVOS.forEach(optMonitoringVO -> {
            String crossId = optMonitoringVO.getId();
            for (CrossLastOptResultDTO optResultDTO : optResultDTOS) {
                String id = optResultDTO.getId();
                if (StringUtils.equals(id, crossId)) {
                    Date optTime = optResultDTO.getOptTime();
                    Integer duration = optResultDTO.getDuration();
                    String type = optResultDTO.getType();
                    // 优化时间计算
                    optMonitoringVO.setOptTime(optTime);
                    optMonitoringVO.setTypeDesc(EventInfoTypeEnum.getOptDesc(type));
                    if (Objects.nonNull(optTime)) {
                        // 空放优化中，6秒钟结束，当前时间小于开始时间+持续时间
                        if (Objects.equals(EventInfoTypeEnum.PHASE_EMPTY.getEventType(), type) && (current - optTime.getTime() > 6 * 1000)) {
                            optMonitoringVO.setOptStatus(1);
                            optMonitoringVO.setOptStatusDesc("优化中");
                            // 其他事件优化中，当前时间小于优化时间+持续时间
                        } else if (current - optTime.getTime() <= duration * 1000) {
                            optMonitoringVO.setOptStatus(1);
                            optMonitoringVO.setOptStatusDesc("优化中");
                        }
                    }
                }
            }
        });
        return optMonitoringVOS;
    }

    @Override
    public List<OptMonitoringVO> eventAlarmRealTimeList() throws Exception {
        List<OptMonitoringVO> results = new ArrayList<>(80);
        List<OptMonitoringVO> greenList = crossOptMonitoringList();
        List<OptMonitoringVO> crossList = greenOptMonitoringList();
        results.addAll(greenList);
        results.addAll(crossList);
        List<OptMonitoringVO> sort = results.stream().sorted(Comparator.comparing(OptMonitoringVO::getStartTime).reversed()).collect(Collectors.toList());
        return sort;
    }
}